# Generated by Django 3.1.5 on 2021-09-29 12:01

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('dkron', '0001_initial_20210729'),
    ]

    operations = [
        migrations.AddField(
            model_name='job',
            name='retries',
            field=models.IntegerField(default=0),
        ),
    ]
